% Compute LPC coefficients and visualize the spectral envelope
clc
clear
close all

[x,fs] = wavread('a075Leaf.wav');

% Focus on steady-state vowel segment
x = x(7000:14000);

nfft = 2^10;
nlpc = 12;

%% Generate LPC spectral envelope

fft_x = fft(x,nfft);
[a e] = lpc(x,nlpc);

% Computing the LPC spectral envelope using lecture notes
f = linspace(0,fs,nfft);
den = 0;
for k=2:nlpc+1
  den = den-a(k)*exp(-2*pi*j*(k-1)*f/fs);
end
lpc_x1 = 1./(1-den) * sqrt(e*length(x)); % The term sqrt(e*length(x)) accounts for the power

% Another way of computing the LPC spectral envelope
lpc_x2 = freqz(1,a,nfft/2,fs) * sqrt(e*length(x));

% And a third way of computing the LPC spectral envelope
z = fft(a,nfft);
lpc_x3 = 1./z * sqrt(e*length(x));

figure
plot(log(abs(fft_x(1:nfft/2))), 'k');
hold on
plot(log(abs(lpc_x1(1:nfft/2))), 'ro');
plot(log(abs(lpc_x2)), 'g.');
plot(log(abs(lpc_x3(1:nfft/2))), 'b')
axis tight

%% Compute residual and resynthesize utterance

r     = filter(a,1,x);
xhat  = filter(1,a,r);

figure
subplot(211)
hold on
plot(xhat, 'g.');
plot(x); 
plot(r, 'r');
axis tight

subplot(212)
plot(r, 'r');
axis tight

soundsc(x,fs)
pause(1)
soundsc(xhat,fs)

%% Replace residual with noise 
rn   = rand(size(r));
x_uv = filter(1,a,rn);
pause(1)
soundsc(x_uv,fs)

%% Replace residual with pulse train

r1 = zeros(300,1);
r1(1) = 1;
r1 = repmat(r1,100,1);

x_v = filter(1,a,r1);
pause(1)
soundsc(x_v,fs)





